//////////////////////////////////////////////////////////////////////
// INCLUSIONS
//////////////////////////////////////////////////////////////////////
#include "main.h"
#include "sfx.h"
#include "sprites.h"
#include "windows.h"

// BACKGROUNDS
#include "background.h"

//////////////////////////////////////////////////////////////////////
// VARIABLES GLOBALES
//////////////////////////////////////////////////////////////////////

#define  tiles_WIDTH        64
#define  tiles_HEIGHT       64 
#define  xwing_WIDTH        64
#define  xwing_HEIGHT       64 

// voir main.h
extern const unsigned char map[];
extern const u16 retaliatorPalette[];
extern const u16 retaliatorData[];
extern const u16 tilesPalette[];
extern const u16 tilesData[];
extern const u16 xwingPalette[];
extern const u16 xwingData[];

/////////////////////
// SPRITES
/////////////////////

// voir sprites.h

//////////////////////////////////////////////////////////////////////
// PROTOTYPES
//////////////////////////////////////////////////////////////////////

void displayPicture(const u16[], const u16[]);
void getInput();
void initGame();
void introduction();
void startGame();

//////////////////////////////////////////////////////////////////////
// VARIABLES
//////////////////////////////////////////////////////////////////////

u16 alpha;
pSprite xwing;
pBackground bg2;

//////////////////////////////////////////////////////////////////////
// FONCTIONS
//////////////////////////////////////////////////////////////////////

int main()
{
  // page d'accueil
  introduction();

  // boucle du jeu
  startGame();

  return 0;

} // fin main()

//////////////////////////////////////////////////////////////////////
// Introduction du jeu
//
//////////////////////////////////////////////////////////////////////

void introduction()
{
  // mode 4
  REG_DISPCNT = MODE_4 | BG2_ENABLE;

  // fond
  displayPicture(retaliatorData, retaliatorPalette);

  // on attend le fameux "press start"
  while (*KEYS & KEY_START);

  // fondu au noir
  fadeOut();

} // fin introduction()

//////////////////////////////////////////////////////////////////////
// Lance le jeu
//
//////////////////////////////////////////////////////////////////////

void startGame()
{
  // charge les sprites, etc...
  initGame();

  // alpha blending
  alpha = 8;
  REG_COLEV = BLEND(16, alpha);
  // en spcifiant OBJ comme premire cible de l'alpha blending
  // tous les sprites se verraient affects
  // lorsque TARGET1_OBJ n'est pas prcis, seuls les sprites en
  // MODE_ALPHA sont affects
  REG_BLDMOD = /* TARGET1_OBJ | */ TARGET2_BG2 | SFX_ALPHA;

  // boucle principale
  while (1)
  {
    // v-sync
    waitForVSync();
    // copie les sprites
    copyOAMBuffer();

    ////////////////////////////////

    // gestion des touches
    getInput();
    // dplace le sprite
    moveSprite(xwing, xwing->x, xwing->y);
    // rotation et mise  l'chelle
    rotateSprite(xwing, xwing->angle, xwing->xscale, xwing->yscale);
  }

  delete bg2;
  delete xwing;

} // fin startGame()

//////////////////////////////////////////////////////////////////////
// Gre l'appui sur les touches
//
//////////////////////////////////////////////////////////////////////

void getInput()
{
  // dplacement vers le bas
  if (!(*KEYS & KEY_DOWN) &&
       (xwing->y < SCREEN_HEIGHT - xwing_HEIGHT))
  {
    xwing->y++;
  }

  // dplacement vers la droite
  if (!(*KEYS & KEY_RIGHT) &&
       (xwing->x < SCREEN_WIDTH - xwing_WIDTH))
    xwing->x++;

  // dplacement vers le haut
  if (!(*KEYS & KEY_UP) && (xwing->y > 0))
    xwing->y--;

  // dplacement vers la gauche
  if (!(*KEYS & KEY_LEFT) && (xwing->x > 0))
    xwing->x--;

  // on augmente l'opacit
  if (!(*KEYS & KEY_R) && (alpha < 16))
  {
    alpha++;
    REG_COLEV = BLEND(16, alpha);
  }

  // on diminue l'opacit
  if (!(*KEYS & KEY_L) && (alpha > 0))
  {
    alpha--;
    REG_COLEV = BLEND(16, alpha);
  }

  // rotation vers la droite
  if (!(*KEYS & KEY_B))
  {
    xwing->angle++;
    if (xwing->angle >= 360)
      xwing->angle = 0;
  }

  // rotation vers la gauche
  if (!(*KEYS & KEY_A))
  {
    xwing->angle--;
    if (xwing->angle < 0)
      xwing->angle = 359;
  }

} // fin getInput()

//////////////////////////////////////////////////////////////////////
// Affiche une image plein cran en mode 4.
//
// param : u16[] picture,       l'image
// param : u16[] pictpal,       la palette de couleurs
//
//////////////////////////////////////////////////////////////////////

void displayPicture(const u16 picture[], const u16 pictpal[])
{
  // copie de la palette de couleurs
  // XXX : utilisation du DMA
  for (int i = 0; i < 256; i++)
    palette[i] = pictpal[i];

  // affichage de l'image
  REG_DMA3SAD = (u32) picture;
  REG_DMA3DAD = (u32) videoBuffer;
  REG_DMA3CNT = SCREEN_HEIGHT * SCREEN_WIDTH / 2 | DMA_16NOW;

} // fin displayPicture()

//////////////////////////////////////////////////////////////////////
// Initialise les donnes des tiles et de la map.
//
//////////////////////////////////////////////////////////////////////

void initBackground()
{
  // initialisation du background
  bg2 = new Background;
  bg2->number = 2;
  bg2->charBaseBlock = 3;
  bg2->screenBaseBlock = 31;
  bg2->colorMode = BG_COLOR_256;
  bg2->size = ROTBG_SIZE_512x512;
  bg2->mosaic = 0;

  enableBackground(bg2);

  // copie de la palette des tiles
  for (int i = 0; i < 256; i++)
    palette[i] = tilesPalette[i];

  // copie des donnes des tiles
  REG_DMA3SAD = (u32) tilesData;
  REG_DMA3DAD = (u32) bg2->tileData;
  REG_DMA3CNT = tiles_WIDTH * tiles_HEIGHT | DMA_16NOW;
  //for (int i = 0; i < tiles_WIDTH * tiles_HEIGHT / 2; i++)
  //  bg2->tileData[i] = tilesData[i];

  u16* temp = (u16 *) map;
  for (int i = 0; i < 64 * 64 / 2; i++)
    bg2->mapData[i] = temp[i];

} // fin initBackground()

//////////////////////////////////////////////////////////////////////
// Initialise les donnes du programme.
//
//////////////////////////////////////////////////////////////////////

void initGame()
{
  // mode 4
  REG_DISPCNT = MODE_2 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_1D;

  // sprites
  // initAngles();
  initSprites();

  // background
  initBackground();

  // palette de couleur des sprites
  for (int i = 0; i < 256; i++)
    spritesPalette[i] = xwingPalette[i];

  // spriteID = 0
  // rotationID = 0
  xwing = createSprite(0, 0);
  xwing->x = 0;
  xwing->y = 0;

  // donnes des sprites
  // en mode bitmap (mode 4),
  // les 512 premiers octets ne sont pas disponibles
  REG_DMA3SAD = (u32) xwingData;
  REG_DMA3DAD = (u32) (&OAMData[512 * 16]);
  // 256 * 8 :
  // nous avons un sprite de 64x64 pixels, divis en tiles
  // de 8x8 pixels, donc nous avons 8x8 tiles.
  // un tiles contient 8x8 pixels
  // 8x8x8 = 512 octets pour une ligne de tiles
  // il y a 8 lignes de tiles
  // comme nous copions 2 octets  la fois, nous copions 256*8
  // fois 16 bits
  REG_DMA3CNT = 256 * 8 | DMA_16NOW;

  //for (int i = xwing->spriteID; i < 256 * 8; i++)
  //  OAMData[i + 512 * 16] = xwingData[i];

  // initialisation du premier sprite
  xwing->oam->attribute0 = COLOR_256 | /* SIZE_DOUBLE | */ MODE_ALPHA |
                           ROTATION_FLAG | SQUARE | xwing->y;
  xwing->oam->attribute1 = SIZE_64 | ROTDATA(xwing->rotationID) | xwing->x;
  xwing->oam->attribute2 = xwing->spriteID + 512; // mode bitmap

} // fin initGame()

// Fin de main.cpp
